@@ -22,9 +22,9 @@
 
 #include "../common/types.h"
 #include "../common/iobuf.h"
+#include "../common/util.h"
 #include "keydb.h"
 #include "keyedit.h"
-#include "../common/util.h"
 
 /* It could be argued that the default cipher should be 3DES rather
    than AES128, and the default compression should be 0
@@ -351,8 +351,10 @@ gpg_error_t parse_and_set_import_filter (const char *string);
 import_filter_t save_and_clear_import_filter (void);
 void            restore_import_filter (import_filter_t filt);
 
-gpg_error_t read_key_from_file (ctrl_t ctrl, const char *fname,
-                                kbnode_t *r_keyblock);
+gpg_error_t read_key_from_file_or_buffer (ctrl_t ctrl, const char *fname,
+                                          const void *buffer, size_t buflen,
+                                          kbnode_t *r_keyblock);
+gpg_error_t import_included_key_block (ctrl_t ctrl, kbnode_t keyblock);
 void import_keys (ctrl_t ctrl, char **fnames, int nnames,
 		  import_stats_t stats_hd, unsigned int options,
                   int origin, const char *url);
@@ -371,6 +373,7 @@ struct impex_filter_parm_s
 {
   ctrl_t ctrl;
   kbnode_t node;
+  char hexfpr[2*MAX_FINGERPRINT_LEN + 1];
 };
 
 const char *impex_filter_getval (void *cookie, const char *propname);
@@ -394,6 +397,8 @@ void export_print_stats (export_stats_t stats);
 
 int parse_export_options(char *str,unsigned int *options,int noisy);
 gpg_error_t parse_and_set_export_filter (const char *string);
+void push_export_filters (void);
+void pop_export_filters (void);
 
 int exact_subkey_match_p (KEYDB_SEARCH_DESC *desc, kbnode_t node);
 
@@ -406,6 +411,7 @@ int export_secsubkeys (ctrl_t ctrl, strlist_t users, unsigned int options,
 
 gpg_error_t export_pubkey_buffer (ctrl_t ctrl, const char *keyspec,
                                   unsigned int options,
+                                  const void *prefix, size_t prefixlen,
                                   export_stats_t stats,
                                   kbnode_t *r_keyblock,
                                   void **r_data, size_t *r_datalen);
